@@ -1,7 +1,6 @@
 #!/usr/bin/env python3
 
-
-'''
+"""
 
 DragonGen.py
 
@@ -22,8 +21,11 @@ Some guidelines for work on this file moving forward:
   - Comment any lines of code that are confusing
   - Don't code-golf
 
+Things to keep in mind when working on this file:
 
-'''
+
+
+"""
 
 import traceback
 import platform
@@ -36,17 +38,12 @@ from .variable_types import ProjectVars
 from .util import *
 from buildgen.generator import BuildFileGenerator
 
-
 # Rules and defaults
 _LAZY_RULES_DOT_YML: dict = None
 _LAZY_DEFAULTS_DOT_YML: dict = None
 
 _IS_THEOS_MAKEFILE_ = False
 
-
-# Ninja Statements
-# TODO: move to types file
-
 # These are used like so:
 # a_build_object = Build("output files here", "rule name here", "input files here")
 # outputs = a_build_object.outputs
@@ -70,20 +67,20 @@ class Generator(object):
         self.config: dict = config
         self.module_name: str = module_name
         self.target_platform: str = target_platform
-
+        self.project_variables: ProjectVars = None
 
     def write_output_file(self, stream: TextIO):
-        '''
+        """
         Evaluate outline with variables and write ninja file to given IO stream
 
         Keyword arguments:
         stream -- IO stream to which the ninja data should be writen
-        '''
-        
+        """
+
         # Compute project variables
         self.project_variables: ProjectVars = ProjectVars(
-                                self.generate_vars(self.config[self.module_name], self.target_platform))
-        
+            self.generate_vars(self.config[self.module_name], self.target_platform))
+
         # Generate the outline
         outline = self.generate_ninja_outline()
 
@@ -103,18 +100,17 @@ class Generator(object):
                 continue
             if isinstance(item, Rule):
                 gen.rule(item.name,
-                        description=item.description,
-                        command=item.command)
+                         description=item.description,
+                         command=item.command)
                 continue
-            if isinstance(item, Build): 
+            if isinstance(item, Build):
                 gen.build(item.outputs, item.rule, item.inputs)
                 continue
             if isinstance(item, Default):
                 gen.default(['$build_target_file'])
 
-
     def generate_vars(self, module_variables: dict, target: str) -> dict:
-        '''
+        """
         Generate ProjectVars object for a project
 
         Keyword arguments:
@@ -122,7 +118,7 @@ class Generator(object):
         target -- target platform
 
         Raises: KeyError
-        '''
+        """
 
         if 'for' in module_variables:
             target = module_variables['for']
@@ -134,22 +130,22 @@ class Generator(object):
 
         if _IS_THEOS_MAKEFILE_:
             project_dict.update({
-                        'theosshim': '-include$$DRAGONDIR/include/PrefixShim.h -w'
-                        })
+                'theosshim': '-include$$DRAGONDIR/include/PrefixShim.h -w'
+            })
 
         # Setup with default vars
         project_dict.update(get_default_section_dict('Defaults'))  # Universal
         try:
             # Apply Type Variables
-            project_dict.update(get_default_section_dict('Types', 
-                                module_variables['type'], 'variables'))  # Type-based
+            project_dict.update(get_default_section_dict('Types',
+                                                         module_variables['type'], 'variables'))  # Type-based
         except KeyError as ex:
             try:
                 # Cast type to lowercase and try again
                 project_dict.update(get_default_section_dict('Types',
-                                    module_variables['type'].lower(), 'variables'))
+                                                             module_variables['type'].lower(), 'variables'))
             except KeyError:
-                # They either didn't include a type variable, or they misspelled the 
+                # They either didn't include a type variable, or they misspelled the
                 #     type they used.
                 raise ex
 
@@ -164,12 +160,13 @@ class Generator(object):
         for source in get_default_section_dict(), module_variables, self.config:
             if 'all' in source:
                 project_dict.update(source['all'])
-            if 'Targets' in source and target in source['Targets']:
+            if 'Targets' in source and target in source['Targets'] and 'all' in source['Targets'][target]:
                 project_dict.update(source['Targets'][target]['all'])
 
+        project_dict.update(module_variables)
 
         # MACHINE checks
-        for d,i in enumerate(project_dict['archs']):
+        for d, i in enumerate(project_dict['archs']):
             if 'MACHINE' in i:
                 project_dict['archs'][d] = platform.machine()
 
@@ -177,7 +174,6 @@ class Generator(object):
             project_dict['triple'] = '-target ' + os.popen('clang -print-target-triple').read().strip() \
                 if 'MACHINE' in project_dict['triple'] else '-target ' + project_dict['triple']
 
-        
         # A few variables that need to be renamed
         NINJA_KEYS = {
             'location': 'install_location',
@@ -187,41 +183,39 @@ class Generator(object):
             'lopt': 'lopts'
         }
         # Rename them
-        project_dict.update({key: project_dict[NINJA_KEYS[key]] 
-                            for key in NINJA_KEYS 
-                                if NINJA_KEYS[key] in project_dict})
-
+        project_dict.update({key: project_dict[NINJA_KEYS[key]]
+                             for key in NINJA_KEYS
+                             if NINJA_KEYS[key] in project_dict})
 
         project_dict['lowername'] = str(project_dict['name']).lower()
 
         # Apply framework/lib search and additional search dirs
         project_dict['fwSearch'] = project_dict['fw_dirs'] \
-                                    + (project_dict['additional_fw_dirs'] 
-                                        if project_dict['additional_fw_dirs'] 
-                                        else [])
+                                   + (project_dict['additional_fw_dirs']
+                                      if project_dict['additional_fw_dirs']
+                                      else [])
         project_dict['libSearch'] = project_dict['lib_dirs'] \
-                                    + (project_dict['additional_lib_dirs'] 
-                                        if project_dict['additional_lib_dirs'] 
-                                        else [] )
-
+                                    + (project_dict['additional_lib_dirs']
+                                       if project_dict['additional_lib_dirs']
+                                       else [])
 
         # Specify toolchain paths
         # TODO: maybe we can use `find` to track down the binaries and figure out prefixes?
         if len(os.listdir(os.environ['DRAGONDIR'] + '/toolchain')) > 1:
             project_dict.update({k: f'$dragondir/toolchain/linux/iphone/bin/'
-                + project_dict[k] for k in [
-                'cc',
-                'cxx',
-                'lipo',
-                'dsym',
-                'plutil',
-                'swift',
-                'ld',
-            ]})
+                                    + project_dict[k] for k in [
+                                     'cc',
+                                     'cxx',
+                                     'lipo',
+                                     'dsym',
+                                     'plutil',
+                                     'swift',
+                                     'ld',
+                                 ]})
             project_dict.update({k: '$dragondir/toolchain/linux/iphone/bin/'
-                + project_dict[k] for k in [
-                    'codesign',
-            ]})
+                                    + project_dict[k] for k in [
+                                     'codesign',
+                                 ]})
 
         # TODO: lazy hack
         if 'cxxflags' in project_dict:
@@ -233,40 +227,36 @@ class Generator(object):
 
         if 'name_override' in project_dict:
             project_dict['name'] = project_dict['name_override']
-        
-        if os.environ['DGEN_DEBUG']:
-            pprint("project_dict after processing through generate_vars:" + str(project_dict), stream=sys.stderr)
-            print("\n\n", file=sys.stderr)
+
+        log.debug("project_dict after processing through generate_vars:" + log.format(project_dict))
 
         return project_dict
 
-
     def rules_and_build_statements(self) -> (list, list):
-        '''
+        """
         Generate build statements and rules for a given variable set.
 
         Returns rule_list, build_state as extensions for an outline
-        '''
+        """
 
         # Trivial project types
         if self.project_variables['type'] == 'resource-bundle':
             return [
-                    get_rule('bundle'),
-                    get_rule('stage'),
-                ], [
-                    Build('bundle', 'bundle', 'build.ninja'),
-                    Build('stage', 'stage', 'build.ninja'),
-                ]
+                       get_rule('bundle'),
+                       get_rule('stage'),
+                   ], [
+                       Build('bundle', 'bundle', 'build.ninja'),
+                       Build('stage', 'stage', 'build.ninja'),
+                   ]
         if self.project_variables['type'] == 'stage':
             return [
-                    get_rule('stage'),
-                ], [
-                    Build('stage', 'stage', 'build.ninja'),
-                ]
-
+                       get_rule('stage'),
+                   ], [
+                       Build('stage', 'stage', 'build.ninja'),
+                   ]
 
         # Only load rules we need
-        FILE_RULES = {  
+        FILE_RULES = {
             'c_files': 'c',
             'cxx_files': 'cxx',
             'dlists': None,
@@ -294,7 +284,7 @@ class Generator(object):
                 name, ext = os.path.split(f)[1], os.path.splitext(f)[1]
                 if ext == '.x':
                     build_state.append(Build(f'$builddir/logos/{name}.m', 'logos', f))
-                    filedict.setdefault('objc_files', []) # Create a list here if it doens't exist
+                    filedict.setdefault('objc_files', [])  # Create a list here if it doens't exist
                     filedict['objc_files'].append(f'$builddir/logos/{name}.m')
                 elif ext == '.xm':
                     build_state.append(Build(f'$builddir/logos/{name}.mm', 'logos', f))
@@ -327,23 +317,23 @@ class Generator(object):
                 cmd = rules(f'archive{a}', 'cmd') + ' ' + ' '.join(linker_conds)
                 rule_list.append(Rule(f'link{a}', rules(f'link{a}', 'desc'), cmd))
                 build_state.append(Build(f'$builddir/$name.{a}',
-                                        f'link{a}',
-                                        arch_specific_object_files))
+                                         f'link{a}',
+                                         arch_specific_object_files))
             else:
                 # Linker rules and build statements
                 cmd = rules(f'link{a}', 'cmd') + ' ' + ' '.join(linker_conds)
                 rule_list.append(Rule(f'link{a}', rules(f'link{a}', 'desc'), cmd))
                 build_state.append(Build(f'$builddir/$name.{a}',
-                                        f'link{a}',
-                                        arch_specific_object_files))
+                                         f'link{a}',
+                                         arch_specific_object_files))
 
         build_state.extend([
             # lipo if needed, else use a dummy rule to rename it to what the next rule expects
             # the dummy rule could be optimized out, but its probably more developmentally clear
             #       to have it there anyways /shrug
             Build('$internalsymtarget',
-                'lipo' if len(self.project_variables['archs']) > 1 else 'dummy',
-                [f'$builddir/$name.{a}' for a in self.project_variables['archs']]),
+                  'lipo' if len(self.project_variables['archs']) > 1 else 'dummy',
+                  [f'$builddir/$name.{a}' for a in self.project_variables['archs']]),
             # Debug symbols
             Build('$internalsigntarget', 'debug', '$internalsymtarget'),
             # Codesign
@@ -361,13 +351,12 @@ class Generator(object):
 
         return rule_list, build_state
 
-
     def generate_ninja_outline(self) -> list:
-        '''
+        """
         Generate list of unevaluated build.ninja statements
 
         Seealso: rules_and_build_statements
-        '''
+        """
 
         outline = [
             Var('name'),
@@ -412,13 +401,13 @@ class Generator(object):
             Var('swift'),
             ___,
             Var('targetvers'),
-            Var('targetprefix'),
             Var('targetos'),
             Var('triple'),
             ___,
             Var('frameworks'),
             Var('libs'),
             ___,
+            Var('prefix'),
             Var('macros'),
             Var('arc'),
             Var('btarg'),
@@ -429,7 +418,7 @@ class Generator(object):
             Var('warnings'),
             ___,
             Var('cinclude'),
-            Var('header_includes'), 
+            Var('header_includes'),
             Var('public_headers'),
             ___,
             Var('usrCflags'),
@@ -466,11 +455,11 @@ class Generator(object):
 
 
 def rules(*key_path: str) -> dict:
-    '''
+    """
     Lazy load default rules and return value specified path.
 
     Raises: FileNotFoundError, KeyError
-    '''
+    """
 
     global _LAZY_RULES_DOT_YML
     if _LAZY_RULES_DOT_YML is None:
@@ -486,11 +475,11 @@ def rules(*key_path: str) -> dict:
 
 
 def get_default_section_dict(*key_path: str) -> dict:
-    '''
+    """
     Lazy load defaults.yml and return the requested dictionary from it
 
     Raises: FileNotFoundError, KeyError
-    '''
+    """
 
     global _LAZY_DEFAULTS_DOT_YML
     with open(f'{os.environ["DRAGONDIR"]}/internal/defaults.yml') as f:
@@ -508,25 +497,28 @@ def get_default_section_dict(*key_path: str) -> dict:
 
 
 def handle(ex: Exception):
-    ''' Optionally print debug information '''
+    """ Optionally print debug information """
 
     dberror("Press v for detailed debugging output, any other key to exit.")
 
-    old_setting = termios.tcgetattr(sys.stdin.fileno())
-    tty.setraw(sys.stdin)
-    x = sys.stdin.read(1)
-    termios.tcsetattr(0, termios.TCSADRAIN, old_setting)
-    if str(x).lower() == 'v':
-        dberror(str(ex))
-        dberror(''.join(traceback.format_tb(ex.__traceback__)))
-    else:
+    try:
+        old_setting = termios.tcgetattr(sys.stdin.fileno())
+        tty.setraw(sys.stdin)
+        x = sys.stdin.read(1)
+        termios.tcsetattr(0, termios.TCSADRAIN, old_setting)
+        if str(x).lower() == 'v':
+            dberror(str(ex))
+            dberror(''.join(traceback.format_tb(ex.__traceback__)))
+        else:
+            dberror("Exiting...")
+    except Exception:
         dberror("Exiting...")
 
     print(f'export DRAGONGEN_FAILURE=1')
 
 
 def main():
-    '''
+    """
     Generate and write build.ninja file from DragonMake or Makefile
 
     Outline of this method:
@@ -534,8 +526,8 @@ def main():
         - Process raw data if needed via the Makefile or Legacy (bash) interpreters
         - Iterate through the top-level keys in the dictionary
         - Call the Generator class to write to {top-level-key-name}.ninja for each
-        - Pass some export commands to the parent bash script via stdout 
-    '''
+        - Pass some export commands to the parent bash script via stdout
+    """
     META_KEYS = {  # Keys that may be at the root of the DragonMake dict
         'name': 'package_name',
         'icmd': 'install_command',
@@ -557,6 +549,10 @@ def main():
         'package': None,
         'desc': None,
         'all': None,
+        'icon': None,
+        'depiction': None,
+        'sldepiction': None,
+        'sileodepiction': None
     }
 
     exports = {}
@@ -571,7 +567,7 @@ def main():
                 # If the file we tried to load isn't YAML, try running it as a bash script
                 if os.system("sh DragonMake 2>/dev/null") == 0:
                     # If that worked, it's the old (OLD) legacy DragonMake format,
-                    #   which we can easily support via a couple lines of regex 
+                    #   which we can easily support via a couple lines of regex
                     config = load_old_format(open('DragonMake'))
                     dbstate("Loading Legacy format DragonMake")
                 else:
@@ -580,9 +576,10 @@ def main():
                     dberror("Check YAML syntax or file an issue")
                     raise ex
 
-
     elif os.path.exists('Makefile'):
-        config = interpret_theos_makefile(open('Makefile'))
+        from .theos import TheosMakefileProcessor
+        config = TheosMakefileProcessor().project
+        # config = interpret_theos_makefile(open('Makefile'))
         exports['theos'] = 1
         dbstate("Generating build scripts from Theos Makefile")
         global _IS_THEOS_MAKEFILE_
@@ -590,12 +587,12 @@ def main():
 
     else:
         raise FileNotFoundError
-    
+
     dbstate("Generating build scripts")
     for key in config:
         if key in META_KEYS:
             continue
-        
+
         # Hack to run a bash command in the context of DragonGen from a DragonMake file
         # TODO: remove this when the main dragon script is pythonized
         if key == 'exports':
@@ -612,7 +609,7 @@ def main():
             # if i add a key to control.py and don't add it to meta tags here, this happens
             # so maybe find a better way to do that, dpkg is complex and has many fields
             dbwarn("! Warning: Key %s is not a valid module (a dictionary),"
-                " nor is it a known configuration key" % key)
+                   " nor is it a known configuration key" % key)
             dbwarn("! This key will be ignored.")
             continue
 
@@ -644,9 +641,10 @@ def main():
         print(f'export {x}="{exports[x]}"')
 
 
-
 if __name__ == '__main__':
     try:
+        if os.environ['DGEN_DEBUG']:
+            log.LOG_LEVEL = LogLevel.DEBUG
         main()
     except FileNotFoundError as exception:
         print('Error: No project files found', file=sys.stderr)
